"""
slick.py is a library for talking to slick (http://code.google.com/p/slickqa).

Copyright 2013 AccessData Group, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

from . import micromodels


class Configuration(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    configurationType = micromodels.StringField()
    filename = micromodels.StringField()

    # Using BaseField for now as a way of holding a dictionary field
    configurationData = micromodels.BaseField()

    def create_reference(self):
        reference = ConfigurationReference()
        reference.configId = self.id
        reference.name = self.name
        if hasattr(self, 'filename'):
            reference.filename = self.filename
        return reference


class Link(micromodels.Model):
    name = micromodels.StringField()
    url = micromodels.StringField()


class StoredFile(micromodels.Model):
    id = micromodels.StringField()
    filename = micromodels.StringField()
    chunkSize = micromodels.IntegerField()
    uploadDate = micromodels.DateTimeField(use_int=True)
    mimetype = micromodels.StringField()
    md5 = micromodels.StringField()
    length = micromodels.IntegerField()


class ConfigurationReference(micromodels.Model):
    configId = micromodels.StringField()
    name = micromodels.StringField()
    filename = micromodels.StringField()


class ReloadStatus(micromodels.Model):
    systemName = micromodels.StringField()
    reloadTime = micromodels.DateTimeField(use_int=True)
    systemStatus = micromodels.StringField()


class Quote(micromodels.Model):
    id = micromodels.StringField()
    quote = micromodels.StringField()
    imageUrl = micromodels.StringField()
    attributed = micromodels.StringField()


class Build(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    built = micromodels.DateTimeField(use_int=True)
    description = micromodels.StringField()

    def create_reference(self):
        ref = BuildReference()
        ref.buildId = self.id
        ref.name = self.name
        return ref


class BuildReference(micromodels.Model):
    buildId = micromodels.StringField()
    name = micromodels.StringField()


class Release(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    target = micromodels.DateTimeField(use_int=True)
    defaultBuild = micromodels.StringField()
    builds = micromodels.ModelCollectionField(Build)
    status = micromodels.StringField()

    def create_reference(self):
        ref = ReleaseReference()
        ref.releaseId = self.id
        ref.name = self.name
        return ref


class ReleaseReference(micromodels.Model):
    releaseId = micromodels.StringField()
    name = micromodels.StringField()


class FeatureReference(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()


class Feature(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    description = micromodels.StringField()
    imgUrl = micromodels.StringField()
    img = micromodels.ModelField(StoredFile)

    def create_reference(self):
        ref = FeatureReference()
        ref.id = self.id
        ref.name = self.name
        return ref


class ComponentReference(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    code = micromodels.StringField()


class Component(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    description = micromodels.StringField()
    code = micromodels.StringField()
    features = micromodels.ModelCollectionField(Feature)

    def create_reference(self):
        ref = ComponentReference()
        ref.id = self.id
        ref.name = self.name
        if hasattr(self, 'code') and self.code is not None:
            ref.code = self.code
        return ref


class DataDrivenPropertyType(micromodels.Model):
    name = micromodels.StringField()
    requirement = micromodels.BooleanField()
    standardValues = micromodels.FieldCollectionField(micromodels.StringField)


class Project(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    description = micromodels.StringField()
    configuration = micromodels.ModelField(Configuration)
    defaultRelease = micromodels.StringField()
    releases = micromodels.ModelCollectionField(Release)
    inactiveReleases = micromodels.ModelCollectionField(Release)
    lastUpdated = micromodels.DateTimeField(use_int=True)
    tags = micromodels.FieldCollectionField(micromodels.StringField())
    attributes = micromodels.BaseField()
    automationTools = micromodels.FieldCollectionField(micromodels.StringField())
    components = micromodels.ModelCollectionField(Component)
    datadrivenProperties = micromodels.ModelCollectionField(DataDrivenPropertyType)

    def create_reference(self):
        reference = ProjectReference()
        reference.id = self.id
        reference.name = self.name
        return reference


class ProjectReference(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()


class ProductVersion(micromodels.Model):
    """This class represents a version of a product.  The only product included with slick for now is 'slick'"""
    productName = micromodels.StringField()
    versionString = micromodels.StringField()


class TestcaseQuery(micromodels.Model):
    queryDescription = micromodels.StringField()
    className = micromodels.StringField()

# TODO: define all the types of test case queries

class NamedTestCaseQuery(micromodels.Model):
    name = micromodels.StringField()
    query = micromodels.ModelField(TestcaseQuery)


class Step(micromodels.Model):
    name = micromodels.StringField()
    expectedResult = micromodels.StringField()


class TestcaseReference(micromodels.Model):
    testcaseId = micromodels.StringField()
    name = micromodels.StringField()
    automationId = micromodels.StringField()
    automationKey = micromodels.StringField()
    automationTool = micromodels.StringField()


class Testcase(micromodels.Model):
    id = micromodels.StringField()
    created = micromodels.DateTimeField(use_int=True)
    name = micromodels.StringField()
    purpose = micromodels.StringField()
    requirements = micromodels.StringField()
    steps = micromodels.ModelCollectionField(Step)
    author = micromodels.StringField()
    attributes = micromodels.BaseField()
    automated = micromodels.BooleanField()
    automationPriority = micromodels.IntegerField()
    automationTool = micromodels.StringField()
    automationConfiguration = micromodels.StringField()
    automationId = micromodels.StringField()
    automationKey = micromodels.StringField()
    stabilityRating = micromodels.IntegerField()
    tags = micromodels.FieldCollectionField(micromodels.StringField())
    project = micromodels.ModelField(ProjectReference)
    component = micromodels.ModelField(ComponentReference)
    dataDriven = micromodels.ModelCollectionField(DataDrivenPropertyType)
    deleted = micromodels.BooleanField()

    def create_reference(self):
        reference = TestcaseReference()
        reference.testcaseId = self.id
        reference.name = self.name
        if hasattr(self, 'automationId'):
            reference.automationId = self.automationId
        if hasattr(self, 'automationKey'):
            reference.automationKey = self.automationKey
        if hasattr(self, 'automationTool'):
            reference.automationTool = self.automationTool
        return reference


class Testplan(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    createdBy = micromodels.StringField()
    project = micromodels.ModelField(ProjectReference)
    sharedWith = micromodels.FieldCollectionField(micromodels.StringField())
    isprivate = micromodels.BooleanField()
    queries = micromodels.ModelCollectionField(NamedTestCaseQuery)


class ConfigurationOverride(micromodels.Model):
    key = micromodels.StringField()
    value = micromodels.StringField()
    isRequirement = micromodels.BooleanField()


class TestplanRunParameters(micromodels.Model):
    config = micromodels.ModelField(ConfigurationReference)
    runtimeOptions = micromodels.ModelField(ConfigurationReference)
    release = micromodels.ModelField(ReleaseReference)
    build = micromodels.ModelField(BuildReference)
    configurationOverride = micromodels.ModelCollectionField(ConfigurationOverride)


class ResultsByStatus(micromodels.Model):
    PASS = micromodels.IntegerField()
    FAIL = micromodels.IntegerField()
    BROKEN_TEST = micromodels.IntegerField()
    NOT_TESTED = micromodels.IntegerField()
    SKIPPED = micromodels.IntegerField()
    NO_RESULT = micromodels.IntegerField()
    CANCELLED = micromodels.IntegerField()
    PASSED_ON_RETRY = micromodels.IntegerField()


class TestrunSummary(micromodels.Model):
    totalTime = micromodels.IntegerField()
    resultsByStatus = micromodels.ModelField(ResultsByStatus)
    statusListOrdered = micromodels.FieldCollectionField(micromodels.StringField())
    total = micromodels.IntegerField()


class SlickUpdate(micromodels.Model):
    updateId = micromodels.StringField()
    description = micromodels.StringField()
    name = micromodels.StringField()
    needsApplying = micromodels.BooleanField()


class TestrunReference(micromodels.Model):
    testrunId = micromodels.StringField()
    name = micromodels.StringField()


class Testrun(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    testplanId = micromodels.StringField()
    testplan = micromodels.ModelField(Testplan)
    config = micromodels.ModelField(ConfigurationReference)
    runtimeOptions = micromodels.ModelField(ConfigurationReference)
    project = micromodels.ModelField(ProjectReference)
    dateCreated = micromodels.DateTimeField(use_int=True)
    runStarted = micromodels.DateTimeField(use_int=True)
    runFinshed = micromodels.DateTimeField(use_int=True)
    release = micromodels.ModelField(ReleaseReference)
    build = micromodels.ModelField(BuildReference)
    summary = micromodels.ModelField(TestrunSummary)
    files = micromodels.ModelCollectionField(StoredFile)
    links = micromodels.ModelCollectionField(Link)
    info = micromodels.StringField()
    state = micromodels.StringField()
    attributes = micromodels.BaseField()
    requirements = micromodels.FieldCollectionField(micromodels.StringField())

def create_reference(self):
        ref = TestrunReference()
        ref.testrunId = self.id
        ref.name = self.name
        return ref


class GroupType:
    PARALLEL = "PARALLEL"
    SERIAL = "SERIAL"

    def __init__(self):
        pass


class TestrunGroup(micromodels.Model):
    id = micromodels.StringField()
    name = micromodels.StringField()
    created = micromodels.DateTimeField(use_int=True)
    testruns = micromodels.ModelCollectionField(Testrun)
    groupType = micromodels.StringField()
    groupSummary = micromodels.ModelField(TestrunSummary)


class TestRunParameter(micromodels.Model):
    fulFilledRequirements = micromodels.BaseField()
    automationTool = micromodels.StringField()
    hostname = micromodels.StringField()


class ResultStatus:
    PASS = "PASS"
    FAIL = "FAIL"
    NOT_TESTED = "NOT_TESTED"
    NO_RESULT = "NO_RESULT"
    BROKEN_TEST = "BROKEN_TEST"
    SKIPPED = "SKIPPED"
    CANCELLED = "CANCELLED"
    PASSED_ON_RETRY = "PASSED_ON_RETRY"

    def __init__(self):
        pass


class RunStatus:
    SCHEDULED = "SCHEDULED"
    TO_BE_RUN = "TO_BE_RUN"
    RUNNING = "RUNNING"
    FINISHED = "FINISHED"

    def __init__(self):
        pass


class LogEntry(micromodels.Model):
    entryTime = micromodels.DateTimeField(use_int=True)
    level = micromodels.StringField()
    loggerName = micromodels.StringField()
    message = micromodels.StringField()
    exceptionClassName = micromodels.StringField()
    exceptionMessage = micromodels.StringField()
    exceptionStackTrace = micromodels.FieldCollectionField(micromodels.StringField())


class UpdateRecord(micromodels.Model):
    id = micromodels.StringField()
    updateId = micromodels.StringField()
    logs = micromodels.ModelCollectionField(LogEntry)


class ResultReference(micromodels.Model):
    resultId = micromodels.StringField()
    status = micromodels.StringField()
    recorded = micromodels.DateTimeField(use_int=True)
    build = micromodels.ModelField(BuildReference)


class GraphColumnReference(micromodels.Model):
    type = micromodels.StringField()
    name = micromodels.StringField()


class GraphValueReference(micromodels.Model):
    date = micromodels.DateTimeField(use_int=True)
    measurements = micromodels.FieldCollectionField(micromodels.IntegerField())


class Graph(micromodels.Model):
    columns = micromodels.ModelCollectionField(GraphColumnReference)
    values = micromodels.ModelCollectionField(GraphValueReference)


class Result(micromodels.Model):
    id = micromodels.StringField()
    testrun = micromodels.ModelField(TestrunReference)
    config = micromodels.ModelField(ConfigurationReference)
    configurationOverride = micromodels.ModelCollectionField(ConfigurationOverride)
    testcase = micromodels.ModelField(TestcaseReference)
    recorded = micromodels.DateTimeField(use_int=True)
    started = micromodels.DateTimeField(use_int=True)
    finished = micromodels.DateTimeField(use_int=True)
    status = micromodels.StringField()
    runstatus = micromodels.StringField()
    reason = micromodels.StringField()
    attributes = micromodels.BaseField()
    files = micromodels.ModelCollectionField(StoredFile)
    links = micromodels.ModelCollectionField(Link)
    log = micromodels.ModelCollectionField(LogEntry)
    project = micromodels.ModelField(ProjectReference)
    component = micromodels.ModelField(ComponentReference)
    release = micromodels.ModelField(ReleaseReference)
    build = micromodels.ModelField(BuildReference)
    runlength = micromodels.IntegerField()
    history = micromodels.ModelCollectionField(ResultReference)
    hostname = micromodels.StringField()
    requirements = micromodels.FieldCollectionField(micromodels.StringField())
    graph = micromodels.ModelField(Graph)


class HostStatus(micromodels.Model):
    hostname = micromodels.StringField()
    lastCheckIn = micromodels.DateTimeField(use_int=True)
    currentWork = micromodels.ModelField(Result)


class SystemConfiguration(micromodels.Model):
    """
    This model should be inherited by a sub model as each SystemConfiguration class will have different properties.
    The properties listed here are ones that are global to all SystemConfiguration instances (they are part of the
    java interface).
    """
    id = micromodels.StringField()
    configurationType = micromodels.StringField()
    name = micromodels.StringField()
    className = micromodels.StringField()


class AMQPSystemConfiguration(SystemConfiguration):
    """
    AMQP System Configurations.  The class name and configurationType should be fixed.
    """
    exchangeName = micromodels.StringField()
    hostname = micromodels.StringField()
    port = micromodels.IntegerField()
    username = micromodels.StringField()
    password = micromodels.StringField()
    virtualHost = micromodels.StringField()

    def __init__(self):
        super(AMQPSystemConfiguration, self).__init__()
        self.className = 'org.tcrun.slickij.api.data.AMQPSystemConfiguration'
        self.configurationType = 'amqp-system-configuration'


class EmailTemplateConfiguration(SystemConfiguration):
    """Configuration for email templates.  If the project reference is null, the template is considered global."""
    project = micromodels.ModelField(ProjectReference)
    subjectTemplate = micromodels.StringField()
    emailTemplate = micromodels.StringField()

    def __init__(self):
        super(EmailTemplateConfiguration, self).__init__()
        self.className = 'org.tcrun.slickij.api.data.EmailTemplateConfiguration'
        self.configurationType = 'email-template-configuration'


class EmailSystemConfiguration(SystemConfiguration):
    """Global Slick email configuration settings.  Things like smtp configuration and the sender email address are
    configured here."""

    smtpHostname = micromodels.StringField()
    smtpPort = micromodels.IntegerField()
    smtpUsername = micromodels.StringField()
    smtpPassword = micromodels.StringField()
    ssl = micromodels.BooleanField()
    enabled = micromodels.BooleanField()
    sender = micromodels.StringField()

    def __init__(self):
        super(EmailSystemConfiguration, self).__init__()
        self.className = 'org.tcrun.slickij.api.data.EmailSystemConfiguration'
        self.configurationType = 'email-system-configuration'
        self.configurationName = 'Global Email System Configuration'


class EmailOffSwitch(SystemConfiguration):
    """A way to turn off emails for particular parts of the product"""
    turnOffEmailsForType = micromodels.StringField()
    turnOffEmailsForId = micromodels.StringField()

    def __init__(self):
        super(EmailOffSwitch, self).__init__()
        self.className = 'org.tcrun.slickij.api.data.EmailOffSwitch'
        self.configurationType = 'email-off-switch'


class SubscriptionInfo(micromodels.Model):
    """Data representing an email subscription to events that happen on slick."""

    subscriptionType = micromodels.StringField()
    subscriptionValue = micromodels.StringField()
    onStart = micromodels.BooleanField()


class EmailSubscription(SystemConfiguration):
    """Email subscriptions have the email address as the name, and embed a list of subscription info models"""
    enabled = micromodels.BooleanField()
    subscriptions = micromodels.ModelCollectionField(SubscriptionInfo)

    def __init__(self):
        super(EmailSubscription, self).__init__()
        self.className = 'org.tcrun.slickij.api.data.EmailSubscription'
        self.configurationType = 'email-subscription'


class ComparisonTypes:
    EQUALS_IGNORE_CASE = "equals-ignore-case"
    EQUALS = "equals"
    CONTAINS = "contains"

    def __init__(self):
        pass


class MatchCriteria(micromodels.Model):
    """Match criteria for AutomaticTestrunGroup System Configuration.
    This describes a single match against a testrun."""
    propertyName = micromodels.StringField()
    propertyValue = micromodels.StringField()
    comparisonType = micromodels.StringField()


class AutomaticTestrunGroup(SystemConfiguration):
    enabled = micromodels.BooleanField()
    template = micromodels.StringField()
    groupType = micromodels.StringField()
    replaceSameBuild = micromodels.BooleanField()
    matchers = micromodels.ModelCollectionField(MatchCriteria)

    def __init__(self):
        super(AutomaticTestrunGroup, self).__init__()
        self.configurationType = 'auto-add-to-testrungroup'
        self.replaceSameBuild = True



"""
slick.py is a library for talking to slick (http://code.google.com/p/slickqa).

Copyright 2013 AccessData Group, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
__author__ = 'Jason Corbett'

from . import micromodels

class SlickQuery(micromodels.Model):
    """Base class for all query classes.  Query classes aren't used by slick, but are a way of documenting which
    query options are available for various endpoints."""

    def __init__(self):
        super(SlickQuery, self).__init__()


class ConfigurationQuery(SlickQuery):
    """Configuration Query options.  There are 3:
     * name: You can query for the configuration by Name.
     * configurationType: Although this is a string (on the server side as well), there are a few known configuration
       types: PROJECT, and ENVIRONMENT.  You probably won't query for PROJECT types, as they are automatically included
       in the project itself.
     * filename: This corresponds to the filename parameter of the configuration data type.  It is not directly used
       by the server for anything, but is an additional way to distinguish one configuration from another.
    """
    name = micromodels.StringField()
    configurationType = micromodels.StringField()
    filename = micromodels.StringField()

    def __init__(self, name=None, configurationType=None, filename=None):
        """Create a new configuration query, optionally specifying the parameters"""
        super(ConfigurationQuery, self).__init__()

        if name is not None:
            self.name = name
        if configurationType is not None:
            self.configurationType = configurationType
        if filename is not None:
            self.filename = filename

class SystemConfigurationQuery(SlickQuery):
    """
    Query for SystemConfigurations.  SystemConfigurations are different in that they are only an interface on the server
    and any java class can inherit and become a SystemConfiguration.  Because of this there is no specific model for
    SystemConfiguration.

    The query options are for a specific config-type or name.
    """
    configtype = micromodels.StringField('config-type')
    name = micromodels.StringField()

    def __init__(self, configtype = None, name = None):
        """Query for a system configuration.  You should specify a configuration type, otherwise you will get lots of
        different system configuration types back.
        """
        if configtype is not None:
            self.configtype = configtype
        if name is not None:
            self.name = name

class ResultQuery(SlickQuery):
    testrunid = micromodels.StringField()
    status = micromodels.StringField()
    excludestatus = micromodels.StringField()
    runstatus = micromodels.StringField()
    allfields = micromodels.BooleanField()

    def __init__(self, testrunid=None, status=None, excludestatus=None, runstatus=None, allfields=None):
        if testrunid is not None:
            self.testrunid = testrunid
        if status is not None:
            self.status = status
        if excludestatus is not None:
            self.excludestatus = excludestatus
        if runstatus is not None:
            self.runstatus = runstatus
        if allfields is not None:
            self.allfields = allfields

class TestcaseQuery(SlickQuery):
    projectid = micromodels.StringField()
    componentid = micromodels.StringField()
    automationKey = micromodels.StringField()
    automationId = micromodels.StringField()
    automationTool = micromodels.StringField()
    tag = micromodels.StringField()
    automated = micromodels.BooleanField()
    author = micromodels.StringField()
    namecontains = micromodels.StringField()
    name = micromodels.StringField()

    def __init__(self, projectid=None, componentid=None, automationKey=None, automationId=None, automationTool=None, tag=None, automated=None, author=None, namecontains=None, name=None):
        if projectid is not None:
            self.projectid = projectid
        if componentid is not None:
            self.componentid = componentid
        if automationKey is not None:
            self.automationKey = automationKey
        if automationId is not None:
            self.automationId = automationId
        if automationTool is not None:
            self.automationTool = automationTool
        if tag is not None:
            self.tag = tag
        if automated is not None:
            self.automated = automated
        if author is not None:
            self.author = author
        if namecontains is not None:
            self.namecontains = namecontains
        if name is not None:
            self.name = name

class TestrunGroupQuery(SlickQuery):
    createdafter = micromodels.DateTimeField(use_int=True)
    name = micromodels.StringField()

    def __init__(self, createdafter=None, name=None):
        if createdafter is not None:
            self.createdafter = createdafter
        if name is not None:
            self.name = name

class TestplanQuery(SlickQuery):
    projectid = micromodels.StringField()
    createdby = micromodels.StringField()

    def __init__(self, projectid, createdby=None):
        self.projectid = projectid
        if createdby is not None:
            self.createdby = createdby

class TestrunQuery(SlickQuery):
    projectid = micromodels.StringField()
    releaseid = micromodels.StringField()
    buildid = micromodels.StringField()
    createdafter = micromodels.DateTimeField(use_int=True)
    configid = micromodels.StringField()
    testplanid = micromodels.StringField()
    configName = micromodels.StringField()
    projectName = micromodels.StringField()
    releaseName = micromodels.StringField()
    buildName = micromodels.StringField()
    name = micromodels.StringField()
    limit = micromodels.IntegerField()

    def __init__(self, projectid=None, releaseid=None, buildid=None, createdafter=None, configid=None, testplanid=None,
                 configName=None, projectName=None, releaseName=None, buildName=None, name=None, limit=None):
        if projectid is not None:
            self.projectid = projectid
        if releaseid is not None:
            self.releaseid = releaseid
        if buildid is not None:
            self.buildid = buildid
        if createdafter is not None:
            self.createdafter = createdafter
        if configid is not None:
            self.configid = configid
        if testplanid is not None:
            self.testplanid = testplanid
        if configName is not None:
            self.configName = configName
        if projectName is not None:
            self.projectName = projectName
        if releaseName is not None:
            self.releaseName = releaseName
        if buildName is not None:
            self.buildName = buildName
        if name is not None:
            self.name = name
        if limit is not None:
            self.limit = limit

class HostStatusQuery(SlickQuery):
    """Query for a hosts status"""
    checkincutoff = micromodels.IntegerField()

    def __init__(self, checkincutoff=None):
        """You can query for all the host status within a time window between "now" and a number of minutes ago.  The
        default cut off is 5 minutes."""
        if checkincutoff is not None:
            self.checkincutoff = checkincutoff



"""
slick.py is a library for talking to slick (http://code.google.com/p/slickqa).

Copyright 2013 AccessData Group, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
__author__ = 'Jason Corbett'

import requests
import logging
import sys
import traceback
import pprint

try:
    from urllib.parse import urlencode, quote
except ImportError:
    from urllib import urlencode, quote

from .micromodels import Model
from .data import *
from . import queries

import os
import mimetypes
import hashlib

if not mimetypes.inited:
    mimetypes.init()
mimetypes.add_type('text/plain', '.log')

json_content = {'Content-Type': 'application/json'}
STREAM_CONTENT = {'Content-Type': 'application/octet-stream'}


class FindOneModeEnum(object):
    """

    """
    FIRST = 1
    LAST = 2

    def __getattr__(self, name):
        if name in FindOneModeEnum.__dict__:
            return FindOneModeEnum.__dict__[name]
        raise AttributeError


FindOneMode = FindOneModeEnum()


class AttributeDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__


def obj_hook_attr_dict(dct):
    return AttributeDict(dct)


class SlickApiPart(object):
    """A class representing part of the slick api"""

    def __init__(self, model, parentPart, name=None):
        self.model = model
        if name is None:
            self.name = model.__name__.lower() + "s"
        else:
            self.name = name

        self.parent = parentPart
        self.logger = logging.getLogger(name=self.get_name())
        self.data = None

    def get_name(self):
        return self.parent.get_name() + "." + self.name

    def find(self, query=None, **kwargs):
        """
        You can pass in the appropriate model object from the queries module,
        or a dictionary with the keys and values for the query,
        or a set of key=value parameters.
        """
        url = self.getUrl()
        if query is not None:
            if isinstance(query, queries.SlickQuery):
                url = url + "?" + urlencode(query.to_dict())
            elif isinstance(query, dict):
                url = url + "?" + urlencode(query)
        elif len(kwargs) > 0:
            url = url + "?" + urlencode(kwargs)

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        for retry in range(3):
            try:
                self.logger.debug("Making request to slick at url %s", url)
                r = requests.get(url)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    retval = []
                    objects = r.json()
                    for dct in objects:
                        retval.append(self.model.from_dict(dct))
                    return retval
                else:
                    self.logger.error("Slick returned an error when trying to access %s: status code %s" % (url, str(r.status_code)))
                    self.logger.error("Slick response: ", pprint.pformat(r))
                    
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.", url)

    query = find

    def findOne(self, query=None, mode=FindOneMode.FIRST, **kwargs):
        """
        Perform a find, with the same options present, but only return a maximum of one result.  If find returns
        an empty array, then None is returned.

        If there are multiple results from find, the one returned depends on the mode parameter.  If mode is
        FindOneMode.FIRST, then the first result is returned.  If the mode is FindOneMode.LAST, then the last is
        returned.  If the mode is FindOneMode.ERROR, then a SlickCommunicationError is raised.
        """
        results = self.find(query, **kwargs)
        if len(results) is 0:
            return None
        elif len(results) is 1 or mode == FindOneMode.FIRST:
            return results[0]
        elif mode == FindOneMode.LAST:
            return results[-1]

    def get(self):
        """Get the specified object from slick.  You specify which one you want by providing the id as a parameter to
        the parent object.  Example:
        slick.projects("4fd8cd95e4b0ee7ba54b9885").get()
        """
        url = self.getUrl()

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        for retry in range(3):
            try:
                self.logger.debug("Making request to slick at url %s", url)
                r = requests.get(url)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    return self.model.from_dict(r.json())
                else:
                    self.logger.debug("Body of what slick returned: %s", r.text)
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.", url)

    def update(self):
        """Update the specified object from slick.  You specify the object as a parameter, using the parent object as
        a function.  Example:
        proj = slick.projects.findByName("foo")
        ... update proj here
        slick.projects(proj).update()
        """
        obj = self.data
        url = self.getUrl()

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        last_stats_code = None
        last_body = None
        for retry in range(3):
            try:
                json_data = obj.to_json()
                self.logger.debug("Making request to slick at url %s, with data: %s", url, json_data)
                r = requests.put(url, data=json_data, headers=json_content)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    return self.model.from_dict(r.json())
                else:
                    last_stats_code = r.status_code
                    last_body = r.text
                    self.logger.warn("Slick status code: %d", r.status_code)
                    self.logger.warn("Body of what slick returned: %s", r.text)
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
                traceback.print_exc()
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.  Last status code: %d, body: %s", url, last_stats_code, last_body)

    put = update

    def create(self):
        """Create the specified object (perform a POST to the api).  You specify the object as a parameter, using
        the parent object as a function.  Example:
        proj = Project()
        ... add project data here
        proj = slick.projects(proj).create()
        """
        obj = self.data
        self.data = None
        url = self.getUrl()

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        for retry in range(3):
            try:
                json_data = obj.to_json()
                self.logger.debug("Making request to slick at url %s, with data: %s", url, json_data)
                r = requests.post(url, data=json_data, headers=json_content)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    return self.model.from_dict(r.json())
                else:
                    self.logger.debug("Body of what slick returned: %s", r.text)
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.", url)

    post = create

    def remove(self):
        """Remove or delete the specified object from slick.  You specify which one you want by providing the id as
        a parameter to the parent object, using it as a function.  Example:
        slick.projects("4fd8cd95e4b0ee7ba54b9885").remove()
        """
        url = self.getUrl()

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        for retry in range(3):
            try:
                self.logger.debug("Making DELETE request to slick at url %s", url)
                r = requests.delete(url)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    return None
                else:
                    self.logger.debug("Body of what slick returned: %s", r.text)
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.", url)

    delete = remove

    def getUrl(self):
        url = self.parent.getUrl() + "/" + self.name
        if self.data is not None:
            if isinstance(self.data, Model) and hasattr(self.data, 'id'):
                url = url + "/" + self.data.id
            else:
                url = url + "/" + str(self.data)
            self.data = None
        return url

    def __call__(self, *args, **kwargs):
        if len(args) > 0:
            self.data = args[0]
        return self


class SlickProjectApiPart(SlickApiPart):
    def __init__(self, parentPart):
        super(SlickProjectApiPart, self).__init__(Project, parentPart)

    def findByName(self, name):
        """Find a project by it's name"""
        self.data = "byname/" + quote(name) + '?quick=true'
        return self.get()


class SystemConfigurationApiPart(SlickApiPart):
    """
    The system-configuration api is different from other apis.  The model for the return type is variable due to it
    being able to store different instances of system configuration classes.
    """

    def __init__(self, parentPart):
        super(SystemConfigurationApiPart, self).__init__(SystemConfiguration, parentPart, 'system-configuration')

    def __call__(self, model, data=None):
        """Make a request for system-configuration, but you not only need to provide the data, but the model that is
        needed for return type.
        """
        self.model = model
        if data is not None:
            return super(SystemConfigurationApiPart, self).__call__(data)
        else:
            return self

    def find(self, query=None, **kwargs):
        instance = self.model()
        if hasattr(instance, 'configurationType') and instance.configurationType is not None:
            kwargs['config-type'] = instance.configurationType
        return super(SystemConfigurationApiPart, self).find(query, **kwargs)

def upload_chunks(url, stored_file, file_like_obj):
    md5 = hashlib.md5()
    bindata = file_like_obj.read(stored_file.chunkSize)
    while bindata:
        if isinstance(bindata, bytes):
            md5.update(bindata)
        elif isinstance(bindata, str):
            md5.update(bindata.encode('ascii', 'ignore'))
        requests.post(url, data=bindata, headers={'Content-Type': 'application/octet-stream'})
        bindata = file_like_obj.read(stored_file.chunkSize)
    stored_file.md5 = md5.hexdigest()

class StoredFileApiPart(SlickApiPart):

    def __init__(self, parentPart):
        super(StoredFileApiPart, self).__init__(StoredFile, parentPart, "files")

    def upload_local_file(self, local_file_path, file_obj=None):
        """Create a Stored File and upload it's data.  This is a one part do it all type method.  Here is what
        it does:
            1. "Discover" information about the file (mime-type, size)
            2. Create the stored file object in slick
            3. Upload (chunked) all the data in the local file
            4. re-fetch the stored file object from slick, and return it
        """
        if file_obj is None and not os.path.exists(local_file_path):
            return
        storedfile = StoredFile()
        storedfile.mimetype = mimetypes.guess_type(local_file_path)[0]
        storedfile.filename = os.path.basename(local_file_path)
        if file_obj is None:
            storedfile.length = os.stat(local_file_path).st_size
        else:
            file_obj.seek(0,os.SEEK_END)
            storedfile.length = file_obj.tell()
            file_obj.seek(0)
        storedfile = self(storedfile).create()
        md5 = hashlib.md5()
        url = self(storedfile).getUrl() + "/addchunk"
        if file_obj is None:
            with open(local_file_path, 'rb') as filecontents:
                upload_chunks(url, storedfile, filecontents)
        else:
            upload_chunks(url, storedfile, file_obj)
        return self(storedfile).update()


class TestrunGroupApiPart(SlickApiPart):

    def __init__(self, parentPart):
        super(TestrunGroupApiPart, self).__init__(TestrunGroup, parentPart)

    def add_testrun(self, testrun):
        id = testrun
        if isinstance(testrun, Testrun):
            id = testrun.id
        url = self.getUrl()

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        for retry in range(3):
            try:
                self.logger.debug("Making request to slick at url %s", url)
                r = requests.post(url + "/addtestrun/" + id)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    return self.model.from_dict(r.json())
                else:
                    self.logger.debug("Body of what slick returned: %s", r.text)
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.", url)

    def remove_testrun(self, testrun):
        id = testrun
        if isinstance(testrun, Testrun):
            id = testrun.id
        url = self.getUrl()

        # hopefully when we discover what problems exist in slick to require this, we can take the loop out
        for retry in range(3):
            try:
                url = url + "/removetestrun/" + id
                self.logger.debug("Making request to slick at url %s", url)
                r = requests.delete(url)
                self.logger.debug("Request returned status code %d", r.status_code)
                if r.status_code is 200:
                    return self.model.from_dict(r.json())
                else:
                    self.logger.debug("Body of what slick returned: %s", r.text)
            except BaseException as error:
                self.logger.warn("Received exception while connecting to slick at %s", url, exc_info=sys.exc_info())
        raise SlickCommunicationError(
            "Tried 3 times to request data from slick at url %s without a successful status code.", url)




class SlickCommunicationError(Exception):
    def __init__(self, *args, **kwargs):
        super(SlickCommunicationError, self).__init__(*args, **kwargs)


class SlickConnection(object):
    """Slick Connection contains the information on how to connect to slick."""
    logger = logging.getLogger("slick.SlickConnection")

    def __init__(self, baseUrl):
        """Create a new connection to slick, providing the base url under which to contact slick."""
        if baseUrl is None or not isinstance(baseUrl, str):
            SlickConnection.logger.error("Base URL provided to slick connection is not a string.")
            raise SlickCommunicationError

        if baseUrl.endswith("/"):
            self.baseUrl = baseUrl + "api"
        else:
            self.baseUrl = baseUrl + "/api"
        self.configurations = SlickApiPart(Configuration, self)
        self.projects = SlickProjectApiPart(self)
        self.projects.releases = SlickApiPart(Release, self.projects)
        self.projects.releases.builds = SlickApiPart(Build, self.projects.releases)
        self.projects.components = SlickApiPart(Component, self.projects)
        self.systemconfigurations = SystemConfigurationApiPart(self)
        self.testplans = SlickApiPart(Testplan, self)
        self.testruns = SlickApiPart(Testrun, self)
        self.version = SlickApiPart(ProductVersion, self, name='version')
        self.testcases = SlickApiPart(Testcase, self)
        self.results = SlickApiPart(Result, self)
        self.testrungroups = TestrunGroupApiPart(self)
        self.hoststatus = SlickApiPart(HostStatus, self, name='hoststatus')
        self.updates = SlickApiPart(SlickUpdate, self, name='updates')
        self.updates.records = SlickApiPart(UpdateRecord, self.updates, name='records')
        self.quotes = SlickApiPart(Quote, self)
        self.files = StoredFileApiPart(self)

    def getUrl(self):
        """This method is used by the slick api parts to get the base url."""
        return self.baseUrl

    def get_name(self):
        return "slick"



import logging
import re
import sys
import time
from datetime import datetime
import types

import docutils.core

from .micromodels.fields import ModelCollectionField
from . import SlickConnection, SlickCommunicationError, Release, Build, BuildReference, Component, ComponentReference, \
    Project, Testplan, Testrun, Testcase, RunStatus, Result, ResultStatus, LogEntry, Configuration, TestrunGroup, \
    TestrunReference, Link


def add_log_entry(self, message, level='DEBUG', loggername='', exceptionclassname='', exceptionmessage='',
                  stacktrace=''):
    entry = LogEntry()
    entry.entryTime = int(round(time.time() * 1000))
    entry.message = str(message)
    entry.level = level
    entry.loggerName = loggername
    entry.exceptionClassName = exceptionclassname
    entry.exceptionMessage = exceptionmessage
    entry.exceptionStackTrace = stacktrace
    if not hasattr(self, 'log'):
        self.log = []
    self.log.append(entry)


def update_result(self):
    self.connection.results(self).update()


def update_testrun(self):
    if hasattr(self, 'summary'):
        del self.summary
    self.connection.testruns(self).update()


def add_file_to_result(self, filename, fileobj=None):
    slickfile = self.connection.files.upload_local_file(filename, fileobj)
    if not hasattr(self, 'files'):
        self.files = []
    self.files.append(slickfile)
    self.update()


def add_link_to_result(self, name, url):
    link = Link()
    link.name = name
    link.url = url
    if not hasattr(self, 'links'):
        self.links = []
    self.links.append(link)
    self.update()


def make_result_updatable(result, connection):
    result.connection = connection
    result.update = types.MethodType(update_result, result)
    result.add_file = types.MethodType(add_file_to_result, result)
    result.add_link = types.MethodType(add_link_to_result, result)
    result.add_log_entry = types.MethodType(add_log_entry, result)


def make_testrun_updatable(testrun, connection):
    testrun.connection = connection
    testrun.update = types.MethodType(update_testrun, testrun)
    testrun.add_file = types.MethodType(add_file_to_result, testrun)
    testrun.add_link = types.MethodType(add_link_to_result, testrun)


class SlickQA(object):
    def __init__(self, url, project_name, release_name, build_name, test_plan=None, test_run=None,
                 environment_name=None, test_run_group_name=None):
        self.logger = logging.getLogger('slick-reporter.Slick')
        self.slickcon = None
        self.is_connected = False
        self.project = None
        self.environment = environment_name
        self.component = None
        self.componentref = None
        self.testplan = test_plan
        self.releaseref = None
        self.release = release_name
        self.build = build_name
        self.buildref = None
        self.testrun = test_run
        self.testrunref = None
        self.testrun_group = test_run_group_name
        self.logqueue = []

        self.init_connection(url)
        if self.is_connected:
            self.logger.debug("Initializing Slick...")
            self.init_project(project_name)
            self.init_release()
            self.init_build()
            self.init_testplan()
            self.init_environment()
            self.init_testrun()
            self.init_testrungroup()
            # TODO: if you have a list of test cases, add results for each with notrun status

    def init_connection(self, url):
        try:
            self.logger.debug("Checking connection to server...")
            self.slickcon = SlickConnection(url)
            successful = self.verify_connection()
            if not successful:
                raise SlickCommunicationError(
                    "Unable to verify connection to {} by trying to access the version api".format(
                        self.slickcon.getUrl()))
            self.is_connected = True
        except SlickCommunicationError as se:
            self.logger.error(se.message)

    def verify_connection(self):
        version = self.slickcon.version.findOne()
        if version:
            self.logger.debug("Successfully connected. Using version {}".format(version))
            return True
        self.logger.debug("Unable to connect. No version available.")
        return False

    def init_project(self, project, create=True):
        self.logger.debug("Looking for project by name '{}'.".format(project))
        try:
            self.project = self.slickcon.projects.findByName(project)
        except SlickCommunicationError as err:
            self.logger.error("Error communicating with slick: {}".format(err.args[0]))
        if self.project is None and create:
            self.logger.error("Unable to find project with name '{}', creating...".format(self.project))
            self.project = Project()
            self.project.name = project
            self.project = self.slickcon.projects(self.project).create()

        assert isinstance(self.project, Project)
        self.logger.info("Using project with name '{}' and id: {}.".format(self.project.name, self.project.id))

    def init_release(self):
        release_name = self.release
        self.logger.debug("Looking for release '{}' in project '{}'".format(release_name, self.project.name))
        if not hasattr(self.project, 'releases'):
            self.project.releases = []
        for release in self.project.releases:
            assert isinstance(release, Release)
            if release.name == release_name:
                self.logger.info("Found Release '{}' with id '{}' in Project '{}'.".format(release.name, release.id,
                                                                                           self.project.id))
                self.release = release
                self.releaseref = release.create_reference()
                break
        else:
            self.logger.info("Adding release {} to project {}.".format(release_name, self.project.name))
            release = Release()
            release.name = release_name
            self.release = self.slickcon.projects(self.project).releases(release).create()
            assert isinstance(self.release, Release)
            self.project = self.slickcon.projects(self.project).get()
            self.releaseref = self.release.create_reference()
            self.logger.info("Using newly created release '{}' with id '{}' in Project '{}'.".format(self.release.name,
                                                                                                     self.release.id,
                                                                                                     self.project.name))

    def init_build(self):
        build_number = self.build
        if not hasattr(self.release, 'builds'):
            self.release.builds = []
        for build in self.release.builds:
            if build.name == build_number:
                self.logger.debug("Found build with name '{}' and id '{}' on release '{}'".format(build.name, build.id,
                                                                                                  self.release.name))
                self.buildref = build.create_reference()
                break
        else:
            self.logger.info("Adding build {} to release {}.".format(build_number, self.release.name))
            build = Build()
            build.name = build_number
            build.built = datetime.now()
            self.buildref = (
                self.slickcon.projects(self.project).releases(self.release).builds(build).create()).create_reference()
            assert isinstance(self.buildref, BuildReference)
            self.logger.info("Using newly created build '{}' with id '{}' in Release '{}' in Project '{}'.".format(
                self.buildref.name, self.buildref.buildId, self.release.name, self.project.name))

    def get_component(self, component_name):
        self.logger.debug(
            "Looking for component with name '{}' in project '{}'".format(component_name, self.project.name))
        for comp in self.project.components:
            if comp.name == component_name:
                assert isinstance(comp, Component)
                self.logger.info(
                    "Found component with name '{}' and id '{}' in project '{}'.".format(comp.name, comp.id,
                                                                                         self.project.name))
                self.component = comp
                self.componentref = self.component.create_reference()
                assert isinstance(self.componentref, ComponentReference)
                return self.component

    def create_component(self, component_name):
        self.logger.info("Adding component {} to project {}.".format(component_name, self.project.name))
        component = Component()
        component.name = component_name
        component.code = component_name.replace(" ", "-")
        self.component = self.slickcon.projects(self.project).components(component).create()
        self.project.components.append(self.component)
        self.componentref = self.component.create_reference()
        self.logger.info("Using newly created component '{}' with id '{}' in project '{}'.".format(
            self.component.name, self.component.id, self.project.name))
        return self.component

    def init_testplan(self):
        if self.testplan:
            testplan_name = self.testplan
            testplan = self.slickcon.testplans.findOne(projectid=self.project.id, name=testplan_name)
            if testplan is None:
                self.logger.debug("Creating testplan with name '{}' connected to project '{}'.".format(testplan_name,
                                                                                                       self.project.name))
                testplan = Testplan()
                testplan.name = testplan_name
                testplan.project = self.project.create_reference()
                testplan.isprivate = False
                testplan.createdBy = "slickqa-python"
                testplan = self.slickcon.testplans(testplan).create()
                self.logger.info("Using newly create testplan '{}' with id '{}'.".format(testplan.name, testplan.id))
            else:
                self.logger.info(
                    "Found (and using) existing testplan '{}' with id '{}'.".format(testplan.name, testplan.id))
            self.testplan = testplan
        else:
            self.logger.warn("No testplan specified for the testrun.")

    def init_environment(self):
        if self.environment is not None:
            env = self.slickcon.configurations.findOne(name=self.environment, configurationType="ENVIRONMENT")
            if env is None:
                env = Configuration()
                env.name = self.environment
                env.configurationType = "ENVIRONMENT"
                env = self.slickcon.configurations(env).create()
            self.environment = env

    def init_testrun(self):
        testrun = Testrun()
        if self.testrun is not None:
            testrun.name = self.testrun
        else:
            if self.testplan is not None:
                testrun.name = self.testplan.name
            else:
                testrun.name = 'Tests run from slick-python'
        if self.testplan is not None:
            testrun.testplanId = self.testplan.id
        testrun.project = self.project.create_reference()
        testrun.release = self.releaseref
        testrun.build = self.buildref
        testrun.state = RunStatus.RUNNING
        testrun.runStarted = int(round(time.time() * 1000))
        if self.environment is not None and isinstance(self.environment, Configuration):
            testrun.config = self.environment.create_reference()

        self.logger.debug("Creating testrun with name {}.".format(testrun.name))
        self.testrun = self.slickcon.testruns(testrun).create()
        make_testrun_updatable(self.testrun, self.slickcon)

    def init_testrungroup(self):
        if self.testrun_group is not None:
            trg = self.slickcon.testrungroups.findOne(name=self.testrun_group)
            if trg is None:
                trg = TestrunGroup()
                trg.name = self.testrun_group
                trg.testruns = []
                trg.created = datetime.now()
                trg = self.slickcon.testrungroups(trg).create()
            self.testrun_group = self.slickcon.testrungroups(trg).add_testrun(self.testrun)

    def add_log_entry(self, message, level='DEBUG', loggername='', exceptionclassname='', exceptionmessage='',
                      stacktrace=''):
        entry = LogEntry()
        entry.entryTime = int(round(time.time() * 1000))
        entry.message = message
        entry.level = level
        entry.loggerName = loggername
        entry.exceptionClassName = exceptionclassname
        entry.exceptionMessage = exceptionmessage
        entry.exceptionStackTrace = stacktrace
        self.logqueue.append(entry)

    def finish_testrun(self):
        assert isinstance(self.testrun, Testrun)
        testrun = Testrun()
        if self.testrun.name:
            testrun.name = self.testrun.name
        else:
            testrun.name = 'Tests run from slick-python'
        testrun.id = self.testrun.id
        testrun.runFinished = int(round(time.time() * 1000))
        testrun.state = RunStatus.FINISHED
        self.logger.debug("Finishing testrun named {}, with id {}.".format(testrun.name, testrun.id))
        self.slickcon.testruns(testrun).update()

    # TODO: need to add logs, files, etc. to a result

    def file_result(self, name, status=ResultStatus.FAIL, reason=None, runlength=0, testdata=None,
                    runstatus=RunStatus.FINISHED, attributes=None, requires=None):
        test = None
        if testdata is not None:
            assert isinstance(testdata, Testcase)
            if testdata.automationId:
                test = self.slickcon.testcases.findOne(projectid=self.project.id, automationId=testdata.automationId)
            if test is None and hasattr(testdata, 'automationKey') and testdata.automationKey is not None:
                test = self.slickcon.testcases.findOne(projectid=self.project.id, automationKey=testdata.automationId)
        if test is None:
            test = self.slickcon.testcases.findOne(projectid=self.project.id, name=name)
        if test is None:
            self.logger.debug("Creating testcase with name '{}' on project '{}'.".format(name, self.project.name))
            test = Testcase()
            if testdata is not None:
                test = testdata
            test.name = name
            test.created = int(round(time.time() * 1000))
            test.project = self.project.create_reference()
            test = self.slickcon.testcases(test).create()
            self.logger.info(
                "Using newly created testcase with name '{}' and id '{}' for result.".format(name, test.id))
        else:
            if testdata is not None:
                # update the test with the data passed in
                assert isinstance(test, Testcase)
                testdata.id = test.id
                testdata.name = name
                testdata.project = self.project.create_reference()
                test = self.slickcon.testcases(testdata).update()
            self.logger.info("Found testcase with name '{}' and id '{}' for result.".format(test.name, test.id))
        result = Result()
        result.testrun = self.testrun.create_reference()
        result.testcase = test.create_reference()
        result.project = self.project.create_reference()
        result.release = self.releaseref
        result.build = self.buildref
        if self.component is not None:
            result.component = self.componentref
        if len(self.logqueue) > 0:
            result.log = []
            result.log.extend(self.logqueue)
            self.logqueue[:] = []
        result.reason = reason
        result.runlength = runlength
        result.end = int(round(time.time() * 1000))
        result.started = result.end - result.runlength
        result.status = status
        result.runstatus = runstatus
        if attributes is not None:
            result.attributes = attributes
        if requires is not None:
            result.requirements = requires
        self.logger.debug("Filing result of '{}' for test with name '{}'".format(result.status, result.testcase.name))
        result = self.slickcon.results(result).create()
        self.logger.info("Filed result of '{}' for test '{}', result id: {}".format(result.status, result.testcase.name,
                                                                                    result.id))
        make_result_updatable(result, self.slickcon)
        return result


class DocStringMetaData(object):
    def __init__(self, func):
        if hasattr(func, '__doc__') and func.__doc__ is not None:
            dom = docutils.core.publish_doctree(func.__doc__).asdom()
            if dom is not None and dom.firstChild is not None and dom.firstChild.nodeName == 'document':
                document = dom.firstChild
                if document.hasChildNodes() and document.firstChild.nodeName == 'paragraph':
                    self.name = document.firstChild.firstChild.nodeValue
                    if len(document.childNodes) > 1:
                        for node in document.childNodes[1:]:
                            self.process_node(node)
                else:
                    self.name = self.get_name_from_function_name(func)
                    for node in document.childNodes:
                        self.process_node(node)
        else:
            self.name = self.get_name_from_function_name(func)

    def get_name_from_function_name(self, func):
        if hasattr(func, '__name__') and func.__name__ is not None and func.__name__ != "":
            s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', func.__name__)
            s2 = re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
            return re.sub(r'_', ' ', re.sub(r'_?[tT]est$', '', re.sub(r'^[Tt]est_?', '', s2))).capitalize()

    def process_node(self, node):
        if node.nodeName == 'block_quote':
            for child_node in node.childNodes:
                self.process_node(child_node)
        if node.nodeName == 'field_list':
            for child_node in node.childNodes:
                self.process_node(child_node)
        if node.nodeName == 'paragraph':
            if hasattr(self, 'purpose'):
                self.purpose = self.purpose + '\n\n' + node.firstChild.nodeValue
            else:
                self.purpose = node.firstChild.nodeValue
        if node.nodeName == 'field':
            if node.firstChild.firstChild.nodeValue == 'expectedResults' and \
                    hasattr(node.childNodes[1].firstChild, 'nodeName') and node.childNodes[1].firstChild.nodeName == 'enumerated_list':
                self.expectedResults = []
                for list_item in node.childNodes[1].firstChild.childNodes:
                    self.expectedResults.append(list_item.firstChild.firstChild.nodeValue)
            elif node.firstChild.firstChild.nodeValue == 'steps' and \
                    hasattr(node.childNodes[1].firstChild, 'nodeName') and node.childNodes[1].firstChild.nodeName == 'enumerated_list':
                self.steps = []
                for list_item in node.childNodes[1].firstChild.childNodes:
                    self.steps.append(list_item.firstChild.firstChild.nodeValue)
            elif node.firstChild.firstChild.nodeValue == 'tags':
                setattr(self, node.firstChild.firstChild.nodeValue,
                        node.childNodes[1].firstChild.firstChild.nodeValue.split(", "))
            else:
                try:
                    setattr(self, node.firstChild.firstChild.nodeValue,
                            node.childNodes[1].firstChild.firstChild.nodeValue)
                except:
                    sys.stderr.write("Problem parsing comment for test: {}".format(self.name))



#!/usr/bin/env python

from setuptools import setup, find_packages

__author__ = 'Jason Corbett'


def get_requirements(filename):
    with open(filename) as f:
        return f.readlines()

setup(
    name="slickqa",
    description="A client library for the Slick QA result database",
    version="2.0.687",
    license="License :: OSI Approved :: Apache Software License",
    long_description=open('README.rst').read(),
    packages=find_packages(),
    package_data={'': ['*.txt', '*.rst', '*.html']},
    include_package_data=True,
    install_requires=get_requirements('requirements.txt'),
    author="Slick Developers",
    url="http://github.com/slickqa/python-client"
)
